;******************************************************************************************************
; Le 05/03/2020 "Animations Flash-Clignt-Feu.asm.asm" sur PIC 16F690  8 Mhz
;
; Version normale (Il existe une version avec des pauses plus longues !).
;
; Montage : Animations_Flash-Clignt-Feu : 
;
; Version pour : Flash + Clignotants clignotants + Feux de passage  niveaux + Soudure  l'arc + Tube fluorescent + Feu de camp.
;
; Site web : http://www.la-tour.info/uts/uts_index.html
; Montage, schma, notice, typon, explication, circuit imprim, programme assembleur microchip pour pic  cette adresse internet.
;
; Montage  PIC 16F690 pour alimenter des charges relies au + 12 volts.
; Sert sur un rseau de train. Rubrique : Modlisme ferroviaire et lectronique locomotive maquette animation et clairage.
;
; Deux groupes de 8 sorties branchs sur 2 ULN2803.
; Un potentiomtre permet de rgler la vitesse des 3 clignotants pour vhicule.
;
; L'entre R3 est relie aux sorties via des diodes et des cavaliers pour choisir la configuration  la mise sous tension :
;
;------ RA5 RA4 RC5 RC4 : Mode (I => Cavalier en place) -------------------------------------
;        4   3   2   1
; RA3 :              0  : Deux clignotants 1,1 Hz progressifs n4 et n5 (4-5 alterns)
; RA3 :              I  : Deux clignotants 1,1 Hz francs n4 et n5 (4-5 alterns)
; RA3 :          0      : Tube fluo n3 + Arc n3
; RA3 :          I      : Feux de pntration n2 et n3
; RA3 :      0          : Flamme n3
; RA3 :      I          : Feu de pntration n1
; RA3 :  0              : Flash photographe n3
; RA3 :  I              : Clignotant n3
;
;********************************************************************************************
;
; L'utilisation des pattes du PIC 16F690
;
; RA0 - Entre Analogique CAN0 (0-5 Volts) = Vitesse d'animation.
; RA3 - Entre RA3 Multiplexe avec les sorties RA5, RA4, RC5 et RC4 via une diode + rsistance de rappel  la masse.
;
; Sorties Groupe 1
;   RA5 - Flamme n1
;   RA4 - Flamme n2
;   RC5 - Flamme n3 - Feu de de pntration n1
;   RC4 - Clignotant n1, 35 sec de fonctionnement + 35 sec de pause
;   RC3 - Clignotant n2, 35 sec de fonctionnement + 35 sec de pause, 10% plus rapide que le n1.
;   RC6 - Flash photographe n1 
;   RC7 - Flash pour radar  n2
;   RB7 - Flash photographe n3 - Clignotant n3, 20% plus rapide que le n1.
;
; Sorties Groupe 2
;   RA1 - Clignotant 1,1 Hz fixe, progressif ou franc n4 (4-5 alterns) passage  niveaux
;   RA2 - Clignotant 1,1 Hz fixe, progressif ou franc n5 (4-5 alterns) passage  niveaux
;   RC0 - Tube fluorescent n1
;   RC1 - Tube fluorescent n2
;   RC2 - Tube fluorescent n3 - Feu de de pntration n2
;   RB4 - Soudure  l'arc n1
;   RB5 - Soudure  l'arc n2
;   RB6 - Soudure  l'arc n3 - Feu de de pntration n3
;
;
; Flammes = Effet de scintillement d'une flamme, divers effets succesifs.
;
; Clignotants n1, n2 et n3 pour vhicule, 35 sec de fonctionnement/35 secondes de pause.
; Clignotants 1,1 Hz progressifs ou francs et alterns n4 et n5 pour passage  niveau avec deux clignotants.
;
; Flash pour photographe, un flash  intervalle alatoire n 1 et n 3
; Flash pour radar, deux flashs raprochs  intervalle alatoire n 2
;
; Tube Fluo 1 = Nombreuses tentatives d'allumages puis reste allum un certain temps.
; Tube Fluo 2 = Nombreuses tentatives d'allumages mais reste allum qu'un petit moment.
; Tube Fluo 3 = Tentatives d'allumages en continu, ne parvient pas  s'allumer.
;
; Soudure  l'arc 1 = Fonctionne 35 secs suivi d'une pause de 25 secs. 0,3  0,8 secs de pause courte, 5  21 flashs de suite
; Soudure  l'arc 2 = Fonctionne 30 secs suivi d'une pause de 40 secs. 0,5    2 secs de pause courte, 10  41 flashs de suite
; Soudure  l'arc 2 = Fonctionne 10 secs suivi d'une pause de 10 secs. 0,5    2 secs de pause courte, 10  41 flashs de suite
;
; Feu de de pntration  n1 pour vhicule, deux flashs rapides raprochs.
; Feux de de pntration n2 et n3 pour vhicule, deux flashs rapides raprochs et alterns entre les deux feux.
;
; Utilise la routine : Calcul de valeur alatoire : http://www.piclist.com/techref/microchip/rand8bit.htm
; Utilise la routine : Calcul de valeur alatoire : https://www.dontronics.com/psbpix/random.html
;
; Mmoire du PIC 16F690 = 4Ko = 0x0000  0x0FFF.
; Page 0 = 0x000  0x7FF, page 0x800  0xFFF.
; Sous programme d'initialisation en 0x900 en page 1, pour laisser de la place en page 0.
;
;
;*******************************************************************
;
	LIST p=16F690, R=DEC ; Utilisation du mode dcimal.
	include	"P16F690.inc"
;
;	Code non protg,  oscill interne sans sortie, pas de dtection tension basse, pas d'utilisation de la patte MCLR, dmarrage lent, Watchdog hors service
	__config (_INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOR_OFF & _IESO_OFF & _FCMEN_OFF)
;
; _INTRC_OSC_NOCLKOUT  EQU  H'3FFC'    ; INTOSCIO oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN
;                                      ; FOSC<2:0> = 100
; _WDT_OFF             EQU  H'3FF7'    ; WDT disabled and can be enabled by SWDTEN bit of the WDTCON register
; _PWRTE_OFF           EQU  H'3FFF'    ; PWRT disabled
; _MCLRE_OFF           EQU  H'3FDF'    ; MCLR pin function is digital input, MCLR internally tied to VDD
; _CPD_OFF             EQU  H'3FFF'    ; Data memory code protection is disabled
; _CP_OFF              EQU  H'3FFF'    ; Program memory code protection is disabled
; _BOREN_OFF           EQU  H'3CFF'    ; BOR disabled
; _IESO_OFF            EQU  H'3BFF'    ; Internal External Switchover mode is disabled
; _FCMEN_OFF           EQU  H'37FF'    ; Fail-Safe Clock Monitor is disabled
;

BANK0	macro			; MACRO : Passer en banque0.
	bcf	STATUS,RP0
	bcf	STATUS,RP1
	endm
BANK1	macro			; Passer en banque1.
	bsf	STATUS,RP0
	bcf	STATUS,RP1
	endm
BANK2	macro			; Passer en banque2.
	bcf	STATUS,RP0
	bsf	STATUS,RP1
	endm

;
;*************************************************************************************************************
;
; Dfinitions des constantes internes modifiables par l'utilisateur.

; Tube Fluo 1 = Cinq tentatives d'allumage puis reste allum un temps relativement long.
VIT_FL1	EQU	7	; Vitesse de base pour le tube fluo 1 (1 = rapide, 7 = moyen, 255 = lent).
POS_FL1	EQU	40	; Dure de pause longue dure du tube fluo 1, par pas de 1 seconde de 1  255, par dfaut = 40 secs.

; Tube Fluo 2 = Nombreuses tentatives d'allumage, mais ne reste allum qu'un court moment.
VIT_FL2	EQU	8	; Vitesse de base pour le tube fluo 2 (1 = rapide, 8 = moyen, 255 = lent).
POS_FL2	EQU	30	; Dure de pause longue dure du tube fluo 2, par pas de 1 seconde de 1  255, par dfaut = 30 secs.

; Tube Fluo 3 = Tentatives d'allumage en continu, mais ne parvient pas  s'allumer.
VIT_FL3	EQU	9	; Vitesse de base pour le tube fluo 3 (1 = rapide, 9 = moyen, 255 = lent).
POS_FL3	EQU	20	; Dure de pause longue dure du tube fluo 3, par pas de 1 seconde de 1  255, par dfaut = 20 secs.

; Soudure  l'arc 1 =	Fonctionne 40 secs suivi d'une pause de 30 secs. De 0,3  0,8 secs de pauses courtes, de 5  21 flashs de suite.
ACT_AR1 EQU	35	; Dure de fonctionnement de l'arc 1,     par pas de 1 seconde de 1  255, par dfaut = 35 secs.
POS_AR1	EQU	25	; Dure de pause longue dure de l'arc 1, par pas de 1 seconde de 1  255, par dfaut = 25 secs.

; Soudure  l'arc 2 =	Fonctionne 30 secs suivi d'une pause de 40 secs. De 0,5  2 secs de pauses courtes, de 10  41 flashs de suite.
ACT_AR2 EQU	30	; Dure de fonctionnement de l'arc 2,     par pas de 1 seconde de 1  255, par dfaut = 30 secs.
POS_AR2	EQU	40	; Dure de pause longue dure de l'arc 2, par pas de 1 seconde de 1  255, par dfaut = 40 secs.

; Soudure  l'arc 3 =	Fonctionne 10 secs suivi d'une pause de 10 secs. De 0,5  2 secs de pauses courtes, de 10  41 flashs de suite.
ACT_AR3 EQU	20	; Dure de fonctionnement de l'arc 3,     par pas de 1 seconde de 1  255, par dfaut = 20 secs.
POS_AR3	EQU	20	; Dure de pause longue dure de l'arc 3, par pas de 1 seconde de 1  255, par dfaut = 20 secs.

; Flash de pntration unique n 1, fonctionne pendant N flashs, puis pause de N secondes.
NB_PE1	EQU	90	; Nb de flashs avant la pause (90 flashs = 30 secondes).
POS_PE1	EQU	15	; Dure de pause longue dure du feu de pntration 1, par pas de 1 seconde de 1  255, par dfaut = 15 secs. 

; Flashs de pntration altrns n 2 et 3, fonctionnent pendant N flashs, puis pause de N secondes.
NB_PE23	EQU	120	; Nb de flashs avant la pause (120 flashs = 40 secondes).
POS_PE23 EQU	20	; Dure de pause longue dure du feu de pntration 2 et 3, par pas de 1 seconde de 1  255, par dfaut = 20 secs. 


; Clignotants rapides pour vhicules. La vitesse de clignotement dpend du potentiomtre et de ces dcalages.
VIT_CLR1 EQU	 8	; Dcalage de vitesse de base pour clignotant 1 (0  50 = lent).
VIT_CLR2 EQU	 4	; Dcalage de vitesse de base pour clignotant 2 (0  50 = lent).
VIT_CLR3 EQU	 0	; Dcalage de vitesse de base pour clignotant 3 (0  50 = lent).

; Clignotants pour passage  niveaux en mode progressif uniquement.
VIT_CLP45 EQU	9	; Vitesse de base pour clignotant en mode progressif (1 = rapide, 9 = moyen=930ms=1,1 hz, 255 = lent).

; Clignotants pour passage  niveaux en mode francs uniquement.
VIT_CLF45 EQU	128	; Vitesse de base pour clignotant en mode fran (1 = rapide, 128 = moyen=930ms=1,1 hz, 255 = lent).



;
;*************************************************************************************************************
;
	CBLOCK 0x20     ; La table des variables commence  0x20.

	cpt_can		; Compteur pour la demande de conversion CAN.
	can_lue1	; Valeur du convertisseur A/N brute = 0-255.
	can_lue2	; Valeur du convertisseur A/N traite = 128+/-64 = 64  192.

	cpt_flam00	; N d'effet en cours.
	cpt_flam01	; Compteur de boucle principale pour l'effet en cours.
	cpt_flam02	; 1er  compteur de dlai en cours.
	cpt_flam03	; 2eme compteur de dlai en cours.

	cpt_flam11	; Puissance flamme n 1.
	cpt_flam12	; Recopie puissance flamme n 1.

	cpt_flam21	; Puissance flamme n 2.
	cpt_flam22	; Recopie puissance flamme n 2.

	cpt_flam31	; Puissance flamme n 3.
	cpt_flam32	; Recopie puissance flamme n 3.

	cpt_lent01	; Compteur de boucle principale.
	cpt_lent02	; Croissance ou dcroissance ou compteur secondaire.
	cpt_lent03	; Temporisation feu teint.
	cpt_lent41	; Puissance feu passage  niveau n 1.
	cpt_lent42	; Puissance feu passage  niveau n 1, recopie.

	cpt_gene00	; Etat gnral de fonctionnement de certaines animation :
			;   Bit(x)=0 => En fonctionnement longue dure.
			;   Bit(x)=1 => En pause longue dure.
			;   Fluo3  = Bit(3), Fluo2  = Bit(2), Fluo1 = Bit(1).
			;   Flash2 = Bit(4).
			;   Clrap3 = Bit(7), Clrap2 = Bit(6), Clrap1 = Bit(5).

	cpt_fluo11	; 1er  Compteur de temps d'activit/inactivit.
	cpt_fluo12	; 2me Compteur de temps d'activit/inactivit.
	cpt_fluo13	; 3me Compteur de temps d'activit/inactivit.
	cpt_fluo14	; Compteur de squence lumineuse pour tube fluorescent. 

	cpt_fluo21	; 1er  Compteur de temps d'activit/inactivit.
	cpt_fluo22	; 2me Compteur de temps d'activit/inactivit.
	cpt_fluo23	; 3me Compteur de temps d'activit/inactivit.
	cpt_fluo24	; Compteur de squence lumineuse pour tube fluorescent. 

	cpt_fluo31	; 1er  Compteur de temps d'activit/inactivit.
	cpt_fluo32	; 2me Compteur de temps d'activit/inactivit.
	cpt_fluo33	; 3me Compteur de temps d'activit/inactivit.
	cpt_fluo34	; Compteur de squence lumineuse pour tube fluorescent. 

	cpt_arc11	; Compteurs de temporisation pour soudure  l'arc 1.
	cpt_arc12	; //
	cpt_arc13	; //
	cpt_arc14	; //
	cpt_arc15	; //
	cpt_arc16	; //
	cpt_arc17	; //
	cpt_arc18	; Etat gnral de fonctionnement de l'arc.

	cpt_arc21	; Compteurs de temporisation pour soudure  l'arc 2.
	cpt_arc22	; //
	cpt_arc23	; //
	cpt_arc24	; //
	cpt_arc25	; //
	cpt_arc26	; //
	cpt_arc27	; //
	cpt_arc28	; Etat gnral de fonctionnement de l'arc.

	cpt_arc31	; Compteurs de temporisation pour soudure  l'arc 3.
	cpt_arc32	; //
	cpt_arc33	; //
	cpt_arc34	; //
	cpt_arc35	; //
	cpt_arc36	; //
	cpt_arc37	; //
	cpt_arc38	; Etat gnral de fonctionnement de l'arc.

	cpt_feup11	; Compteur de boucle principale feux de pntration 1.
	cpt_feup12	; Compteur de boucle secondaire.
	cpt_feup13	; Compteur de squence lumineuse.
	cpt_feup14	; =0 => 1er flash, =1 => 2me flash, =2 => 3me flash
	cpt_feup15	; Compteur de boucle en pause.
	cpt_feup16	; Numro de squence de nombre de flashs.

	cpt_feup21	; Compteur de boucle principale feux de pntration 2 et 3.
	cpt_feup22	; Compteur de boucle secondaire.
	cpt_feup23	; Compteur de squence lumineuse.
	cpt_feup24	; =0 => 1er flash, =1 => 2me flash, =2 => 3me flash
	cpt_feup25	; Compteur de boucle en pause.
	cpt_feup26	; Numro de squence de nombre de flashs.


	cpt_flash11	; Compteur de boucle principale.
	cpt_flash12	; Compteur de boucle secondaire.
	cpt_flash13	; Compteur de boucle tertiaire.

	cpt_flash21	; Compteur de boucle principale.
	cpt_flash22	; Compteur de boucle secondaire.
	cpt_flash23	; Compteur de boucle tertiaire.
			;   bit(0)=0 => 1er  flash voir cpt_gene00(bit 4).
			;   bit(0)=1 => 2me flash

	cpt_flash31	; Compteur de boucle principale.
	cpt_flash32	; Compteur de boucle secondaire.
	cpt_flash33	; Compteur de boucle tertiaire.

	cpt_clrap11	; 
	cpt_clrap12	; 
	cpt_clrap21	; 
	cpt_clrap22	; 
	cpt_clrap31	; 
	cpt_clrap32	; 

	cpt_inter	; Compteur d'interruption pour les sorties PMW.

	temp1		; Variable locale
	temp2		; Variable locale
	mode		; Mode de fonctionnement du programme = Lecture des cavaliers 1  4.

	varale1s : 3	; Variable alatoire sur 16 bits + une variable de combinaison en +3.
	varale2s : 3	; Variable alatoire sur 16 bits + une variable de combinaison en +3.

	w_temp		; Sauvegardes pour les interruptions.
	status_temp	; //

	ENDC
;
;*******************************************************************
; Dbut du programme.
;
	org		0x2100 	  			 ; Initialize EEPROM Data
	dt 		"FLASH_CLIGNT_FEU_V1_Pauses_normales_"	; Texte visible dans le '.hex'.
	dt		"Le 05/03/2020 "
	dt		"http://www.la-tour.info/uts/uts_index.html"
    ORG     0x000
	goto    init
	org     0x004
	goto    interrupt

	dt "__FLASH_CLIGNT_FEU_V1_Pauses_normales__"		; Textes visibles dans le '.hex'.
	dt "__Le 05/03/2020___"
	dt "http://www.la-tour.info/uts/uts_index.html"	;
;
;
;***************************************************************************************
;
; Traitement des interruptions du timer TMR0 pour gnrer les sorties PWM des flammes
; A 8 Mhz Interruption TMR0 = 112 S =  224 instructions dans la boucle principale.
; Comme il y du code dans ce sous programme d'interruption, on passe dans cette routine
; toutes les 117 s + 15 s = 132 s. Ce qui donne pour une rsolution de la PMW sur 128 niveaux = 59 hz.
;
; On charge les valeurs de PMW ici de faon synchrone, pour viter des sursauts de lumire.
; On alimente la led flamme n3 que si le cavalier n3 est en place.
; 
interrupt
	bcf     INTCON,GIE	; Inhibe toutes les interruptions.
	movwf   w_temp  	; Sauve le registre W.
	swapf	STATUS,w	; Swap status avec rsultat dans w.
	movwf	status_temp	; Sauver le STATUS. Pas besoin de passer en BANK0, car on n'utilise ni BANK1 ni BANK2 dans la boucle principale.
	incf	cpt_inter,f	; Compteur pour les flammes 1 + 2 + 3.
	incf	cpt_inter,f
	btfss	STATUS,Z
	goto	chrgpwm
	movf	cpt_flam11,w	; On charge les valeurs quand le compteur = 0,
	movwf	cpt_flam12	; soit toutes les 17 msec = 58 Hz, de faon synchrone
	movf	cpt_flam21,w	; pour viter un sursaut de lumire.
	movwf	cpt_flam22	; RECOPIE => A VOIR SI NECESSAIRE ???
	movf	cpt_flam31,w
	movwf	cpt_flam32
	movf	cpt_lent41,w
	movwf	cpt_lent42
chrgpwm	movf	cpt_inter,w	; Flamme 1 = RA5.
	subwf	cpt_flam12,w	; cpt_flam12 = Valeur de la puissance de la flamme 1 PWM : 0-255, avec une rsolution de 128 niveaux.
	btfsc	STATUS,C
	bsf	PORTA,5		; Allume la flamme 1 PWM.
	btfss	STATUS,C
	bcf	PORTA,5		; Eteint la flamme 1 PWM.
	movf	cpt_inter,w	; Flamme 2 = RA4.
	subwf	cpt_flam22,w	; cpt_flam22 = Valeur de la puissance de la flamme 2 PWM : 0-255, avec une rsolution de 128 niveaux.
	btfsc	STATUS,C
	bsf	PORTA,4		; Allume la flamme 2 PWM.
	btfss	STATUS,C
	bcf	PORTA,4		; Eteint la flamme 2 PWM.
	btfsc	mode,2		; Cavalier[3]=0 ? => Flamme 3
	goto	pasfla3		; Cavalier = 1 !
	movf	cpt_inter,w	; Flamme 3 = RC5.
	subwf	cpt_flam32,w	; cpt_flam32 = Valeur de la puissance de la flamme 3 PWM : 0-255, avec une rsolution de 128 niveaux.
	btfsc	STATUS,C
	bsf	PORTC,5		; Allume la flamme 3 PWM.
	btfss	STATUS,C
	bcf	PORTC,5		; Eteint la flamme 3 PWM.

pasfla3	btfsc	mode,0		; Cavalier[0]=0 ? => Feux lents 4 et 5
	goto	pasfe45		; Cavalier = 1 !
	movf	cpt_inter,w	; Feu clignotant lent 4. cpt_inter = 0, 2, 4, 8,,, 254.
	btfsc	STATUS,Z
	goto	pasfe45		; Si cpt_inter=0 => On saute la squence pour garder les anciens tats en sortie. Permet d'teindre compltement les lampes.
	subwf	cpt_lent42,w	; cpt_lent42 = Valeur de la puissance du feu n 4 PWM : 0-255, avec une rsolution de 128 niveaux.
	btfsc	STATUS,C
	bsf	PORTA,1		; Allume le feu clignotant n 4 PWM.
	btfsc	STATUS,C
	bcf	PORTA,2		; Eteint le feu clignotant n 5 PWM.
	btfss	STATUS,C
	bcf	PORTA,1		; Eteint le feu clignotant n 4 PWM.
	btfss	STATUS,C
	bsf	PORTA,2		; Allume le feu clignotant n 5 PWM.

pasfe45	movlw   199		; Dure = (255-XXX)*temps_us du Timer0 (=112S/2S) = 56.   255-56 = 199.
	movwf   TMR0		; Interruption toutes les 112s*2 = 224 instructions  partir d'ici.
	bcf     INTCON,T0IF	; Remise  0 du drapeau de dpassement du Timer0 pour la prochaine interruption.
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W. W restaur sans modifier status
	retfie  		; Retour d'interruption.
;
;
;********************************************************************************************
;
;	Squence d'inititialisation du PIC ici.
; 	Suite de cette squence en 0x900, pour ne pas encombrer la plage en BANK0 de 0x000  x0x4FF.
;
init	bsf	PCLATH,3	; Prpare le saut vers BANK1 (0x800 - 0xFFF).
	call	init_suite	; Saut en BANK1.
	bcf	PCLATH,3	; On reste ensuite toujours en BANK0 (0x000 - 0x7FF) pour les CALL !
	movlw   199		; Dure = (255-XXX)*temps_us du Timer0 (=112S/2S) = 56.   255-56 = 199.
	movwf   TMR0		; Interruption toutes les 112s*2 = 224 instructions  partir d'ici.
	bcf     INTCON,T0IF	; Remise  0 du drapeau de dpassement du Timer0 pour la prochaine interruption.
	bsf     INTCON,GIE	; L'initialisation est finie, on autorise toutes les interruptions.
;
; Boucle principale
; --------------------------------------------------------------------------------------------
; On restera en BANK0, on ne passe plus en BANK1 ou BANK2 dans cette boucle principale.
; --------------------------------------------------------------------------------------------
;
boucle	
	decfsz	cpt_can,f	; On ne lit le potentiomtre qu'une fois sur 256 = 42 msec.
	goto	t_debut

	btfsc	ADCON0,GO       ; Nouveau calcul de cptlen = Lecture valeur du potentiomtre entre 0 et 255.	
	goto	$-1		; Normalement la dernire conversion est finie depuis longtemps.
	movf	ADRESH,w
	sublw	255 		; Inverse la valeur, car le potentiomtre est cbl  l'envers sur le circuit imprim.
	movwf	can_lue1  	; Vitesse = can_lue1 => 0-255.
	movwf	can_lue2
	sublw	255
	btfss	STATUS,Z
	incf	can_lue1,f   	; Vitesse = can_lue1 => 1-255, pour faire fonctionner les boucles 'decfsz' correctement.
	bcf	STATUS,C
	rrf	can_lue2,f	; can_lue2 de 0  127.
	movlw	64
	addwf	can_lue2,f	; Vitesse = can_lue2 => 64  192.
	bsf	ADCON0,GO       ; Relance la prochaine conversion can.
	movlw	45		; 45 = 50-5 => Boucle de 90 s de temporisation au total avec conversion CAN.
	goto	t_debut+1	; pour quilibrer le temps perdu.

t_debut	movlw	50		; 50 => Boucle de 100 s de temporisation.
	movwf	temp1		; Permet d'tablir un nombre moyen d'instruction par tour de programme.
	decfsz	temp1,f		; Boucle de X tours pour temporiser un peu le programme ici, histoire 
	goto	$-1		; d'quilibrer les retards suivants les passages dans les diffrentes boucles.

;********************************************************************************************************;
; Calcul des valeurs alatoires sur 16 bits pour les squences lumineuses.
; Initialiser les registres 'varale1s' et 'varale2s' avec une valeur diffrente de zro sur 16 bits.
; Source web : https://www.dontronics.com/psbpix/random.html
;            : http://www.joescat.com/micro/pic/an544_rng.html
; 16bit Pseudo Squence Random Number Generator (RNG).
; Based on Microchip's MATH routine found in AN544 and modified to fit all picchips including the tiny 12bit versions.
; This version uses 10 program words, making it slightly faster, but it needs a temp scratch pad to work.
; This RNG is based on linear shift register feedback. Sequence is generated by (Q15 xor Q14 xor Q12 xor Q3).
;
; Ne pas modifier les variables 'varale1s' et 'varale1s+1', autre part dans le programme.
; Utiliser les variables 'varale1s' et 'varale1s+1' sans les modifier.
;
	movf	varale1s,w	; First, ensure that hiB and lowB aren't
	iorwf	varale1s+1,w	; all zeros. If they are, NOT hiB to FFh.
	btfsc	STATUS,Z	; Otherwise, leave hiB and lowB as is.
	comf	varale1s,f
	movlw	0x80		; We want to XOR hiB.7, hiB.6, hiB.4
	btfsc	varale1s,6	; and lowB.3 together in W. Rather than
	xorwf	varale1s,f	; try to line up these bits, we just
	btfsc	varale1s,4	; check to see whether a bit is a 1. If it
	xorwf	varale1s,f	; is, XOR 80h into hiB. If it isn't,
	btfsc	varale1s+1,3	; do nothing. When we're done, the
	xorwf	varale1s,f	; XOR of the 4 bits will be in hiB.7.
	rlf	varale1s,w	; Move hiB.7 into carry.
	rlf	varale1s+1,f	; Rotate c into lowB.0, lowB.7 into c.
	rlf	varale1s,f	; Rotate c into hiB.0.
	movf	varale1s+1,w
	xorwf	varale1s,w	; 'w' = 'varale1s' + 'varale1s+1'
	movwf	varale1s+2

	movf	varale2s,w	; First, ensure that hiB and lowB aren't
	iorwf	varale2s+1,w	; all zeros. If they are, NOT hiB to FFh.
	btfsc	STATUS,Z	; Otherwise, leave hiB and lowB as is.
	comf	varale2s,f
	movlw	0x80		; We want to XOR hiB.7, hiB.6, hiB.4
	btfsc	varale2s,6	; and lowB.3 together in W. Rather than
	xorwf	varale2s,f	; try to line up these bits, we just
	btfsc	varale2s,4	; check to see whether a bit is a 1. If it
	xorwf	varale2s,f	; is, XOR 80h into hiB. If it isn't,
	btfsc	varale2s+1,3	; do nothing. When we're done, the
	xorwf	varale2s,f	; XOR of the 4 bits will be in hiB.7.
	rlf	varale2s,w	; Move hiB.7 into carry.
	rlf	varale2s+1,f	; Rotate c into lowB.0, lowB.7 into c.
	rlf	varale2s,f	; Rotate c into hiB.0.
	movf	varale2s+1,w
	xorwf	varale2s,w	; 'w' = 'varale2s' + 'varale2s+1'
	movwf	varale2s+2

clil_debut
	btfsc	mode,0		; Cavalier[0]=0 ? => Clignotants passage  niveaux.
	goto	clr45_debut	; Cavalier = 1 !
; ------------------------------------------------------------------------------------------
; Squence clignotants 1,1 Hz progressifs et alterne n 4 et n 5 pour passage  niveaux.
; 
; cpt_lent01 = Compteur de boucle principale. 
; cpt_lent02 = Croissance ou dcroissance : Bit(0) => 0 = croit, 1 = dcroit.
; cpt_lent03 = Temporisation feu teint.
; cpt_lent41 = Valeur de puissance calcule PMW 0-255.
; Mise  jour des sorties des leds dans la squence d'interruption.
;
; RA1 = LED 1
; RA2 = LED 2
; ------------------------------------------------------------------------------------------
;
	decfsz	cpt_lent01,f
	goto	flash1_debut
	movlw	VIT_CLP45	; Par dfaut = 9.  cpt_lent01 de 9  0 => 1,5 ms ou de 50  0 => 8 ms.
	movwf	cpt_lent01	; 2 * 9 * 165 s = Clignotants de 0,9 seconde = 1,1 Hz.
	decfsz	cpt_lent03,f    ; cpt_lent03 de 1  0, ou de 70  0.
	goto	flash1_debut
	incf	cpt_lent03,f    ; Par dfaut remet cpt_lent03  1, pour passer directement la prochaine fois.
	btfsc	cpt_lent02,0
	goto	clil_11
	incf	cpt_lent41,f	; cpt_lent02[0]=0 => Lumire croissante (0->255).
	incfsz	cpt_lent41,w	; Pour tester avant que la valeur passe de 255  0.
	goto	flash1_debut
	bsf	cpt_lent02,0
	movlw	70		; 70 pour attendre pendant 100 ms avec un feu compltement teint et un feu compltement allum.
	movwf	cpt_lent03	; Dans ce cas, on fait une pause quand la valeur = 255.
	goto	flash1_debut	; pour que la lampe reste teinte un petit moment.
clil_11	decfsz	cpt_lent41,f	; cpt_lent02[0]=1 => Lumire dcroissante (255->0).
	goto	flash1_debut
	bcf	cpt_lent02,0
	movlw	70		; 70 pour attendre pendant 100 ms avec un feu compltement allum et un feu compltement teint.
	movwf	cpt_lent03
	goto	flash1_debut

clr45_debut			; Cavalier[0]=1 !  => Clignotants rapides.
; -------------------------------------------------------------------------------
; Squence clignotants 1,1 Hz francs et alterns n 4 et n 5.
; 
; cpt_lent01 = Compteur de boucle principale. 
; cpt_lent02 = Compteur de boucle secondaire.
;
; RA1 = LED 1  Cette led est teinte au dpart.
; RA2 = LED 2  Cette led est allume au dpart aprs la lecture des cavaliers.
; -------------------------------------------------------------------------------
;
	decfsz	cpt_lent01,f
	goto	flash1_debut
	movlw	22
	movwf	cpt_lent01
	decfsz	cpt_lent02,f	; On passe ici toutes les 3,6 ms = 22 * 165 s.
	goto	flash1_debut
	movlw	VIT_CLF45	; Par dfaut = 128 ou utiliser 	movf can_lue2,w.
	movwf	cpt_lent02
	movlw	b'00000110'	; On passe ici toutes les 128 * 22 * 165 S = 450 msec
	xorwf	PORTA,f		; Pour un clignotant de 930 ms = 1,1 sec.


flash1_debut
; -------------------------------------------------------------------------------
; Squences Flashs
; Intervalles alatoires.
;
;  cpt_flash11	; Compteur de boucle principale.
;  cpt_flash12	; Compteur de boucle secondaire.
;  cpt_flash13	; Compteur de boucle tertiaire.
;
;  cpt_flash21	; Compteur de boucle principale.
;  cpt_flash22	; Compteur de boucle secondaire.
;  cpt_flash23	; Compteur de boucle tertiaire.
;  cpt_gene00	;   bit(4)=0 => 1er  flash
;  		;   bit(4)=1 => 2me flash
;
;   RC6 - Flash n1 photographe - tempo alatoire comprise entre  6 et 24 sec.
;   RC7 - Flash n2 pour radar  - tempo alatoire comprise entre 12 et 33 sec.
;   RB7 - Flash n3 photographe - tempo alatoire comprise entre  4 et 16 sec.
; -------------------------------------------------------------------------------
;
	decfsz	cpt_flash11	
	goto	flash2_debut
	movlw	7		; On passe ici toutes les 7 * 165 S = 1,1 msec.
	movwf	cpt_flash11
	bcf	PORTC,6		; Eteint la lumire du flash 
	decfsz	cpt_flash12
	goto	flash2_debut
	decfsz	cpt_flash13	; On passe ici toutes les 256 * 7 * 165 S = 295 msec.
	goto	flash2_debut
	movf	varale1s,w
	andlw	0x3F		; 0-63
	addlw	20
	movwf	cpt_flash13	; 20-83 => 6  24 secondes entre deux flashs.
	bsf	PORTC,6		; Allume la lumire du flash pour une dure de 1 msec.
	movlw	2	
	movwf	cpt_flash11	; Pour faire durer le flash 0,3 ms = 2 * 165 S.

flash2_debut
	decfsz	cpt_flash21	
	goto	flash3_debut
	movlw	8		; On passe ici toutes les 8 * 165 S = 1,3 msec.
	movwf	cpt_flash21
	bcf	PORTC,7		; Eteint la lumire du flash 
	btfsc	cpt_gene00,4	
	goto	flah2_1
	decfsz	cpt_flash22
	goto	flash3_debut
	decfsz	cpt_flash23	; On passe ici toutes les 256 * 8 * 165 S = 337 msec.
	goto	flash3_debut
	movf	varale1s+1,w
	andlw	0x3F		; 0-63
	addlw	35
	movwf	cpt_flash23	; 35-98 => 12  33 secondes entre deux flashs.
	bsf	PORTC,7		; Allume la lumire du 1er flash pour une dure de 1 msec.
	bsf	cpt_gene00,4
	movlw	2
	movwf	cpt_flash21	; Pour faire durer le flash 0,3 ms = 2 * 165 S.
	goto	flash3_debut

flah2_1	decfsz	cpt_flash22
	goto	flash3_debut
	bsf	PORTC,7		; Allume la lumire du 2me flash, 337 ms aprs le premier, pour une dure de 1 msec.
	bcf	cpt_gene00,4
	movlw	2	
	movwf	cpt_flash21	; Pour faire durer le flash 0,3 ms = 2 * 165 S.

flash3_debut
	btfsc	mode,3		; Cavalier[3]=0 ? => Flash n3.
	goto	flam1_debut	; Cavalier = 1 !
	decfsz	cpt_flash31	
	goto	flam1_debut
	movlw	9		; On passe ici toutes les 9 * 165 S = 1,4 msec.
	movwf	cpt_flash31
	bcf	PORTB,7
	decfsz	cpt_flash32
	goto	flam1_debut
	decfsz	cpt_flash33	; On passe ici toutes les 256 * 9 * 165 S = 380 msec.
	goto	flam1_debut
	movf	varale2s,w
	andlw	0x1F		; 0-31
	addlw	10
	movwf	cpt_flash33	; 10-41 => 4  16 secondes entre deux flashs.
	bsf	PORTB,7		
	movlw	2	
	movwf	cpt_flash31	; Pour faire durer le flash 0,3 ms = 2 * 165 S.
	goto	flam1_debut

flam1_debut
; ----------------------------------------------------------------------------
; Squence Flammes  2 ou 3 lampes.
; On calcule ici pour les 3 leds.
; Ensuite, dans le sspg d'interruption, on alimente ou non la led n 3 suivant la position du cavalier.
;
; RA5 - Flamme 1 (PWM alatoire)
; RA4 - Flamme 2 (PWM alatoire)
; RC6 - Flamme 3 (PWM alatoire) 
;
; cpt_flam00 = Numro d'effet en cours = 0-10. + suite des effet en n 11 et 13.
; cpt_flam01 = Compteur de boucle principale pour passer N fois dans un effet. 
; cpt_flam02 = 1er  compteur de dlai en cours = 17 => Dcompte les 2,5 msec.
; cpt_flam03 = 2eme compteur de dlai en cours = N * 2,5 msec.
;
; cpt_flam11 = Valeur de puissance calcule PMW 0-255.
; cpt_flam21 = Valeur de puissance recopie dans la routine d'interruption.
; ----------------------------------------------------------------------------
;
	decfsz	cpt_flam02,f
	goto	fluo1_debut
	movlw	17
	movwf	cpt_flam02	; 17 => cpt_flam02, pour avoir cpt_flam03 qui compte les 2,5 millisecondes (17*165s). 
	decfsz	cpt_flam03,f	; Dlai. On passe ici toutes les 2,5 milli-seconde.
	goto	fluo1_debut
	decfsz	cpt_flam01,f	; cpt_flam01 = Compteur de boucles pour un effet.
	goto	fe00
	movf	cpt_flam00,w
	sublw	7
	btfss	STATUS,Z
	goto	f001
	movlw	11		; Si l'effet tait = 7 => On enchaine avec l'effet = 11.
	movwf	cpt_flam00
	goto	f003
f001	movf	cpt_flam00,w
	sublw	9
	btfss	STATUS,Z
	goto	f002
	movlw	13		; Si l'effet tait = 9 => On enchaine avec l'effet = 13.
	movwf	cpt_flam00
	goto	f003
f002	movf	varale1s,w	; Changement alatoire d'effet entre 0 et 10.
	andlw	0x07
	movwf	cpt_flam00	; 0-7
	btfsc	varale1s,5
	incf	cpt_flam00,f	; 0-8
	btfsc	varale1s,6
	incf	cpt_flam00,f	; 0-9
	btfsc	varale1s,7
	incf	cpt_flam00,f	; 0-10
f003	movf	cpt_flam00,w	; 0-10 = cpt_flam00 => N du prochain effet.
	call	table_flam
	movwf	cpt_flam01	; cpt_flam01 = 1-255 => Nb de boucles  raliser pour le prochain effet.
	incf	cpt_flam01,f
	movlw	1
	movwf	cpt_flam03	; 1 => Pour passer dans l'effet directement.
	goto	fluo1_debut
fe00	btfsc	cpt_flam00,0
	goto	fe10
;------------------------------ EFFET 00,02,04,06,08,10 : FLAMMES NORMALES. 80  120 boucles = 7  10 secondes d'animation.
	bcf	STATUS,C	; Puissance = [Ancienne_Puissance + Alatoire(0-127)] / 2
	rrf	varale1s,w	; Pour ne pas toucher  cpt_flam11 lors du calcul intermdiaire.
	movwf	temp1
	bcf	STATUS,C	
	rrf	cpt_flam11,w
	addwf	temp1,w
	movwf	cpt_flam11	; 0-255 => Flamme 1.
	bcf	STATUS,C	; Puissance = [Ancienne_Puissance + Alatoire(0-127)] / 2
	rrf	varale1s+1,w
	movwf	temp1
	bcf	STATUS,C	
	rrf	cpt_flam21,w
	addwf	temp1,w
	movwf	cpt_flam21	; 0-255 => Flamme 2.
	bcf	STATUS,C	; Puissance = [Ancienne_Puissance + Alatoire(0-127)] / 2
	rrf	varale2s,w
	movwf	temp1
	bcf	STATUS,C	
	rrf	cpt_flam31,w
	addwf	temp1,w
	movwf	cpt_flam31	; 0-255 => Flamme 3.
	movf	varale2s+1,w	; Temporisation en 2 * milli-secondes
	andlw	0x3F		; 0-63.
	addlw	10
	movwf	cpt_flam03	; 10-73 => 20  146 msec , Temporisation en fois 2,5 * msec.
	goto	fluo1_debut
fe10	movf	cpt_flam00,w
	sublw	1
	btfss	STATUS,Z
	goto	fe30
;------------------------------ EFFET 01 - cpt_flam00=1 : ROTATION DES 3 FLAMMES. 48 boucles = 10 secondes d'animation.
	movf	cpt_flam11,w
	movwf	temp2
	bcf	STATUS,C	; Puissance = [Ancienne_Puissance + Alatoire(0-127)] / 2
	rrf	varale1s,w	; Pour ne pas toucher  cpt_flam11 lors du calcul intermdiaire.
	movwf	temp1
	bcf	STATUS,C	
	rrf	cpt_flam31,w
	addwf	temp1,w
	movwf	cpt_flam11	; 0-255 => Flamme 1.
	bcf	STATUS,C	; Puissance = [Ancienne_Puissance + Alatoire(0-127)] / 2
	rrf	varale1s+1,w
	movwf	temp1
	bcf	STATUS,C	
	rrf	cpt_flam11,w
	addwf	temp1,w
	movwf	cpt_flam21	; 0-255 => Flamme 2.
	bcf	STATUS,C	; Puissance = [Ancienne_Puissance + Alatoire(0-127)] / 2
	rrf	varale2s,w
	movwf	temp1
	bcf	STATUS,C	
	rrf	temp2,w
	addwf	temp1,w
	movwf	cpt_flam31	; 0-255 => Flamme 3.
	movf	varale2s+1,w	; Temporisation en milli-secondes.
	andlw	0x7F		; 0-127.
	addlw	30
	movwf	cpt_flam03	; 30-157 => 60  314 msec, Temporisation en fois 2,5 * msec.
	goto	fluo1_debut
fe30	movf	cpt_flam00,w
	sublw	3
	btfss	STATUS,Z
	goto	fe50
;------------------------------ EFFET 03 - cpt_flam00=3 : FLAMMES RAPIDES. 60 boucles = 5 secondes d'animation.
	movf	varale1s,w	;
	movwf	cpt_flam11	; 0-255 => Flamme 1.
	movf	varale1s+1,w	;
	movwf	cpt_flam21	; 0-255 => Flamme 2.
	movf	varale2s,w	;
	movwf	cpt_flam31	; 0-255 => Flamme 3.
	movf	varale2s+1,w	; Temporisation en milli-secondes.
	andlw	0x3F		; 0-63.
	addlw	10
	movwf	cpt_flam03	; 10-73 => 20  146 msec, Temporisation en 2,5 * ms.
	goto	fluo1_debut
fe50	movf	cpt_flam00,w
	sublw	5
	btfss	STATUS,Z
	goto	fe70
;------------------------------ EFFET 05 - cpt_flam00=5 : FLAMMES PUISSANTES. 41 boucles = 5 secondes d'animation.
	movf	varale1s,w
	andlw	0x7F		; 0-127
	addlw	100
	movwf	cpt_flam11	; 100-227 => Flamme 1.
	movf	varale1s+1,w
	andlw	0x7F		; 0-127
	addlw	100
	movwf	cpt_flam21	; 100-227 => Flamme 2.
	movf	varale2s,w
	andlw	0x7F		; 0-127
	addlw	100
	movwf	cpt_flam31	; 100-227 => Flamme 3.
	movf	varale2s+1,w	; Temporisation en milli-secondes.
	andlw	0x3F		; 0-36.
	addlw	30
	movwf	cpt_flam03	; 30-93 => 60  183 msec, Temporisation en 2,5 * ms.
	goto	fluo1_debut
fe70	movf	cpt_flam00,w
	sublw	7
	btfss	STATUS,Z
	goto	fe90
;------------------------------ EFFET 07 - cpt_flam00=7 : EXTINCTION PUIS RALLUMAGE LENT(effet n11).  224 boucles =  8 secondes d'animation.
	movf	cpt_flam01,w	; cpt_flam01 = Compteur de boucles pour cet effet 224->0.
fe71	movwf	temp1		; ici w est renseign, on le met dans temp1. Il faut mettre 225 dans la table !
	movf	varale1s,w
	andlw	0x1F		; 0-31
	addwf	temp1,w	; 224+(0-31)  0+(0-31)
	movwf	cpt_flam11	; => Flamme 1.
	movf	varale1s+1,w
	andlw	0x1F		; 0-31
	addwf	temp1,w	; 224 +(0-31)  0+(0-31)
	movwf	cpt_flam21	; => Flamme 2.
	movf	varale2s,w
	andlw	0x1F		; 0-31
	addwf	temp1,w	; 224 +(0-31)  0+(0-31)
	movwf	cpt_flam31	; => Flamme 3.
	movf	cpt_flam00,w
	sublw	7
	btfsc	STATUS,Z
	goto	fe72
	movf	cpt_flam00,w
	sublw	11
	btfsc	STATUS,Z
	goto	fe72	
	movf	varale2s+1,w	; Si effet n 9 ou n 13 on va plus vite.
	andlw	0x07		; 0-7
	addlw	5
	movwf	cpt_flam03	; 5-12 => 10  24 msec, Temporisation en 2,5 * ms.
	goto	fluo1_debut
fe72	movf	varale2s+1,w	; Si effet n 7 ou n 11 on va plus lentement.
	andlw	0x0F		; 0-15
	addlw	10
	movwf	cpt_flam03	; 10-25 => 20  50 msec, Temporisation en 2,5 * ms.
	goto	fluo1_debut
fe90	movf	cpt_flam00,w
	sublw	9
	btfss	STATUS,Z
	goto	feB0
;------------------------------ EFFET 09 - cpt_flam00=9 : EXTINCTION PUIS RALLUMAGE RAPIDE(effet n13).  128 boucles =  3 secondes d'animation.
	movf	cpt_flam01,w	; cpt_flam01 = Compteur de boucles pour cet effet 128->0.
	goto	fe71		; On reprend la squence n 7 avec tempo plus faible en sortie.
feB0	movf	cpt_flam00,w
	sublw	11
	btfss	STATUS,Z
	goto	feD0
;------------------------------ EFFET 11 - cpt_flam00=11 : RALLUMAGE LENT suite effet n 7.
	movf	cpt_flam01,w	; cpt_flam01 = Compteur de boucles pour cet effet 224->0.
	movwf	temp1		; cpt_flam01 de 224  0
	comf	temp1,w		; Donne => temp1 et w de 0  224	
	addlw	-31		; Ici w est renseign.
	goto	fe71		; On reprend la squence n 7  l'envers.
feD0	movf	cpt_flam00,w
	sublw	13
	btfss	STATUS,Z
	goto	fluo1_debut
;------------------------------ EFFET 13 - cpt_flam00=11 : RALLUMAGE RAPIDE suite effet n 9.
	movf	cpt_flam01,w	; cpt_flam01 = Compteur de boucles pour cet effet 128->0.
	movwf	temp1		; cpt_flam01 de 128  0
	comf	temp1,w		; Donne => temp1 et w de 0  128	
	addlw	-128		; Ici w est renseign.
	goto	fe71		; On reprend la squence n 7  l'envers avec tempo plus faible en sortie.


fluo1_debut
; -------------------------------------------------------
; Squence tubes fluorescents.
; cpt_fluo11	; 1er  Compteur de temps actif/inactif.
; cpt_fluo12	; 2me Compteur de temps actif/inactif.
; cpt_fluo13	; 3me Compteur de temps actif/inactif.
; cpt_fluo14	; Compteur de squence lumineuse pour tube fluorescent. 
; cpt_gene00	; Etat gnral de fonctionnement du tube.
; 		; bit(0)=0 => Tube en fonctionnement longue dure
; 		; bit(0)=1 => Tube en pause longue dure
; Des temps d'attente lgrement diffrents dans cpt_fluox2,
; pour viter des allumages synchrones.
;
; Tube Fluo 1 = Cinq tentatives d'allumage puis reste allum un temps relativement long.
; Tube Fluo 2 = Nombreuses tentatives d'allumage, mais ne reste allum qu'un court moment.
; Tube Fluo 3 = Tentatives d'allumage en continu, mais ne parvient pas  s'allumer.
;
; -------------------------------------------------------
; Squence tube fluorescent 1.
;
; RC0 - Tube fluorescent 1
; -------------------------------------------------------
	btfss	cpt_gene00,1	; Tube fluorescent 1 en pose longue dure ?
	goto	f1en_ma

	decfsz	cpt_fluo11,f 	; TUBE EN PAUSE LONGUE DUREE !
	goto	fluo2_debut
	movlw	127
	movwf	cpt_fluo11
	decfsz	cpt_fluo12,f	; On passe ici toutes les 19 ms (128*165 s).
	goto	fluo2_debut

	movlw	47
	movwf	cpt_fluo12

	decfsz	cpt_fluo13,f	; On passe ici toutes les secondes.
	goto	fluo2_debut	; Le temps de pause = cpt_fluo13 (Secondes).
	bcf	cpt_gene00,1	; Retour  la squence => Tube en fonctionnement longue dure.
	movlw	1
	movwf	cpt_fluo11	; cpt_fluoxx=1 => Pour passer directement dans la boucle.
	movwf	cpt_fluo12
	clrf	cpt_fluo14	; Raz du compteur de squence lumineuse.
	goto	fluo2_debut

f1en_ma	decfsz	cpt_fluo11,f	; TUBE EN FONCTIONNEMENT LONGUE DUREE !
	goto	fluo2_debut
	movlw	127
	movwf	cpt_fluo11
	decfsz	cpt_fluo12,f	; On passe ici toutes les 19 ms (127*165 s).
	goto	fluo2_debut
	movlw	VIT_FL1		; 9 = Vitesse moyenne = 0,17 sec.
	movwf	cpt_fluo12	; On passe ici toutes les 0,17 secondes = Pas de lecture de la table 'table_fluo'.
	movlw	0x07
	movwf	PCLATH
	movf	cpt_fluo14,w	; cpt_fluo14 = Compteur de squence de tube fluorescent.
	call	table_fluo	; Valeurs des lampes => temp1.
	movwf	temp1
	btfss	temp1,0		; Tube fluorescent 1.
	bcf	PORTC,0
	btfsc	temp1,0
	bsf	PORTC,0
	incf	cpt_fluo14,f	; cpt_fluo14 = Compteur de squence du tube fluo.
	btfss	temp1,7		; Dernire valeur valide. Fin de table => Bit(7) = 1 ?
	goto	fluo2_debut
	bsf	cpt_gene00,1	; Retour  la squence => Tube en pause longue dure.
	movlw	POS_FL1		; => Temps en seconde de la pause longue dure.
	movwf	cpt_fluo13

fluo2_debut
; -------------------------------------------------------
; Squence tube fluorescent 2.
;
; RC1 - Tube fluorescent 2
; -------------------------------------------------------
	btfss	cpt_gene00,2	; Tube fluorescent 2 en pose longue dure ?
	goto	f2en_ma
	decfsz	cpt_fluo21,f 	; TUBE EN PAUSE LONGUE DUREE !
	goto	fluo3_debut
	movlw	129		; Dures lgrement diffrentes pour viter des boucles synchrones.
	movwf	cpt_fluo21
	decfsz	cpt_fluo22,f	; On passe ici toutes les 17 ms (129*165 s).
	goto	fluo3_debut

	movlw	46
	movwf	cpt_fluo22

	decfsz	cpt_fluo23,f	; On passe ici toutes les secondes.
	goto	fluo3_debut	; Le temps de pause = cpt_fluo23 (Secondes).
	bcf	cpt_gene00,2	; Retour  la squence => Tube en fonctionnement longue dure.
	movlw	1
	movwf	cpt_fluo21	; cpt_fluoxx=1 => Pour passer directement dans la boucle.
	movwf	cpt_fluo22
	clrf	cpt_fluo24	; Raz du compteur de squence lumineuse.
	goto	fluo3_debut
f2en_ma	decfsz	cpt_fluo21,f	; TUBE EN FONCTIONNEMENT LONGUE DUREE !
	goto	fluo3_debut
	movlw	129
	movwf	cpt_fluo21
	decfsz	cpt_fluo22,f	; On passe ici toutes les 17ms (128*165 s).


	goto	fluo3_debut
	movlw	VIT_FL2		; 8 = Vitesse moyenne = 0,17 sec.
	movwf	cpt_fluo22	; On passe ici toutes les 0,17 secondes = Pas de lecture de la table 'table_fluo'.
	movlw	0x07
	movwf	PCLATH
	movf	cpt_fluo24,w	; cpt_fluo24 = Compteur de squence de tube fluorescent.
	call	table_fluo	; Valeurs des lampes => temp1.
	movwf	temp1
	btfss	temp1,1		; Tube fluorescent 2.
	bcf	PORTC,1
	btfsc	temp1,1
	bsf	PORTC,1
	incf	cpt_fluo24,f	; cpt_fluo24 = Compteur de squence du tube fluo.
	btfss	temp1,7		; Dernire valeur valide. Fin de table => Bit(7) = 1 ?
	goto	fluo3_debut
	bsf	cpt_gene00,2	; Retour  la squence => Tube en pause longue dure.
	movlw	POS_FL2		; => Temps en seconde de la pause longue dure.
	movwf	cpt_fluo23

fluo3_debut
	btfsc	mode,1		; Cavalier[1]=0 ? => Fluo 3
	goto	arc1_debut	; Cavalier = 1 !
; -------------------------------------------------------
; Squence tube fluorescent 3.
;
; RC2 - Tube fluorescent 3
; -------------------------------------------------------
	btfss	cpt_gene00,3	; Tube fluorescent 3 en pose longue dure ?
	goto	f3en_ma
	decfsz	cpt_fluo31,f 	; TUBE EN PAUSE LONGUE DUREE !
	goto	arc1_debut
	movlw	130
	movwf	cpt_fluo31
	decfsz	cpt_fluo32,f	; On passe ici toutes les 17 ms (128*165 s).
	goto	arc1_debut

	movlw	45
	movwf	cpt_fluo32

	decfsz	cpt_fluo33,f	; On passe ici toutes les secondes.
	goto	arc1_debut	; Le temps de pause = cpt_fluo33 (Secondes).
	bcf	cpt_gene00,3	; Retour  la squence => Tube en fonctionnement longue dure.
	movlw	1
	movwf	cpt_fluo31	; cpt_fluoxx=1 => Pour passer directement dans la boucle.
	movwf	cpt_fluo32
	clrf	cpt_fluo34	; Raz du compteur de squence lumineuse.
	goto	arc1_debut
f3en_ma	decfsz	cpt_fluo31,f	; TUBE EN FONCTIONNEMENT LONGUE DUREE !
	goto	arc1_debut
	movlw	130
	movwf	cpt_fluo31
	decfsz	cpt_fluo32,f	; On passe ici toutes les 17 ms (128*165 s).
	goto	arc1_debut
	movlw	VIT_FL3		; 7 = Vitesse moyenne = 0,17 sec.
	movwf	cpt_fluo32	; On passe ici toutes les 0,17 secondes = Pas de lecture de la table 'table_fluo'.
	movlw	0x07
	movwf	PCLATH
	movf	cpt_fluo34,w	; cpt_fluo34 = Compteur de squence de tube fluorescent.
	call	table_fluo	; Valeurs des lampes => temp1.
	movwf	temp1
	btfss	temp1,2		; Tube fluorescent 3.
	bcf	PORTC,2
	btfsc	temp1,2
	bsf	PORTC,2
	incf	cpt_fluo34,f	; cpt_fluo34 = Compteur de squence du tube fluo.
	btfss	temp1,7		; Dernire valeur valide. Fin de table => Bit(7) = 1 ?
	goto	arc1_debut
	bsf	cpt_gene00,3	; Retour  la squence => Tube en pause longue dure.
	movlw	POS_FL3		; => Temps en seconde de la pause longue dure.
	movwf	cpt_fluo33


arc1_debut
; -------------------------------------------------------
; Squence arcs lectriques.
;
; cpt_arc11	; 1er  Compteur de temps de temps actif/temps inactif. En marche 40sec / 3mn.
; cpt_arc12	; 2me Compteur de temps de temps actif/temps inactif.
; cpt_arc13	; 3me Compteur de temps de temps actif/temps inactif.
;
; cpt_arc14	; 4me Compteur de temps de temps arrt/nb de squence lumineuse de suite = 10  21.
; cpt_arc15	; 5me Compteur de temps de temps arrt. Arrt entre 2 squences = 0,5  2 secondes.
;
; cpt_arc16	; 6me Compteur de temps Flash teint/lumineux. /Dure d'un flash = 10  101 ms
; cpt_arc17	; 7me Compteur de temps Flash teint/lumineux. \Dure entre deux flashs = 10  31 ms
; cpt_arc18	; Etat gnral de fonctionnement de l'arc.
;
; 		; bit(0)=0 => Flash en pause longue trs dure.
; 		; bit(0)=1 => Flash en fonctionnement trs longue dure.
;
; 		; bit(1)=0 => Flash en arrt courte dure.
; 		; bit(1)=1 => Flash en marche courte dure.
;
; 		; bit(2)=0 => Flash teint.
; 		; bit(2)=1 => Flash lumineux.
;
; Soudure  l'arc 1 = Fonctionne 35 secs suivi d'une pause de 25 secs. De 0,4  0,8 secs de pauses courtes, de 10  41 flashs de suite.
; Soudure  l'arc 2 = Fonctionne 30 secs suivi d'une pause de 40 secs. De 0,3  1,7 secs de pauses courtes, de  5  21 flashs de suite
; Soudure  l'arc 3 = Fonctionne 20 secs suivi d'une pause de 20 secs. De 0,1  1,1 secs de pauses courtes, de  3  18 flashs de suite
;
; -------------------------------------------------------
; Squence arc lectrique 1.
;
; RB4 - Arc lectrique 1.
; -------------------------------------------------------

	btfsc	cpt_arc18,0	; Arc 1 actif/inactif longue dure ?
	goto	a1en_ma

	decfsz	cpt_arc11,f 	; Ici, on est en mode INACTIF TRES LONGUE DUREE.
	goto	arc2_debut
	decfsz	cpt_arc12,f	; On passe ici toutes les 38 ms (256*165 s).
	goto	arc2_debut

	movlw	23
	movwf	cpt_arc12

	decfsz	cpt_arc13,f	; On passe ici toutes les secondes.
	goto	arc2_debut
	movlw	1
	movwf	cpt_arc14
	movwf	cpt_arc15
	movlw	ACT_AR1		; 30 = temps actif => 30 secondes actif.
	movwf	cpt_arc13	; cpt_arcx3 =  Temps du prochain fonctionnement longue dure.
	clrf	cpt_arc18	; Bit(1)=0 =>  Flash en arrt courte dure + Bit(2)=0 => Flash teint.
	bsf	cpt_arc18,0	; Bit(0)=1 =>  Retour  la squence => Arc en fonctionnement longue dure.
	goto	arc2_debut	; On change de mode => Mode actif longue dure (cpt_arc_11, 12, 13, 16, 17).

a1en_ma	decfsz	cpt_arc11,f 	; Ici, on est en mode ACTIF TRES LONGUE DUREE.
	goto	a1en_mb
	decfsz	cpt_arc12,f	; On passe ici toutes les 38 ms (256*165 s).
	goto	a1en_mb

	movlw	23
	movwf	cpt_arc12

	decfsz	cpt_arc13,f	; On passe ici toutes les secondes.
	goto	a1en_mb
	movlw	POS_AR1		; 40 = Temps de pause => 40 secondes inactif.
	movwf	cpt_arc13	; cpt_arcx3 =  Temps de la prochaine pause longue dure.
	bcf	PORTB,4		; On teint le flash.
	bcf	cpt_arc18,0	; Bit(0)=0 =>  Retour  la squence => Arc en pause longue dure.
	goto	arc2_debut	; On change de mode => Mode inactif longue dure (cpt_arc_11, 12, 13).

a1en_mb	btfss	cpt_arc18,1
	goto	a1en_me
	btfsc	cpt_arc18,2
	goto	a1en_md	

	decfsz	cpt_arc16,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ETEINT.
	goto	arc2_debut
	movlw	9
	movwf	cpt_arc16	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc17,f
	goto	arc2_debut
	decfsz	cpt_arc14,f	; Fin de courte squence de flash allum.
	goto	a1en_mc
	movf	varale1s,w	; On a fini les squences de flash.	
	andlw	0x1F
	movwf	cpt_arc15	; 0-31
	movf	varale1s+1,w
	andlw	0x0F		; 0-15
	addlw	16		; 16-31
	addwf	cpt_arc15,f	; 16-62 pour 0,4  2 secondes de pause courte (cpt14+cpt15). (255 = 8 sec).
	bcf	cpt_arc18,1	; Bit(1) = 0. 
	goto	arc2_debut	; On change de mode => Flash en arrt courte dure (cpt_arc_14, 15).

a1en_mc	movf	varale1s,w
	andlw	0x3F		
	movwf	cpt_arc17	; 0-63
	movf	varale1s+1,w
	andlw	0x1F		; 0-31
	addlw	10		; 10-41
	addwf	cpt_arc17,f	; 10-104  cpt_arc17 = 10  104 ms.
	bsf	PORTB,4		; On allume le flash.
	bsf	cpt_arc18,2	; Bit(2) = 1. Flash allume.
	goto	arc2_debut	; On change de mode => Flash allum (cpt_arc_16 + cpt_arc17).

a1en_md	decfsz	cpt_arc16,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ALLUME.
	goto	arc2_debut	
	movlw	9
	movwf	cpt_arc16	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc17,f
	goto	arc2_debut	
	movwf	cpt_arc17	; cpt_arc17 = 10  101 ms.
	movf	varale1s,w
	andlw	0x0F		
	movwf	cpt_arc17	; 0-15
	movf	varale1s+1,w
	andlw	0X07		; 0-7
	addlw	10		; 10-17
	addwf	cpt_arc17,f	; 10-32  cpt_arc17 = 10  32 ms.
	bcf	PORTB,4		; on teint le flash.
	bcf	cpt_arc18,2	; Bit(2) = 0. Flash teint.
	goto	arc2_debut	; On change de mode => Flash teint (cpt_arc_16,17).
	
a1en_me	decfsz	cpt_arc14,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + ARRET COURTE DUREE.
	goto	arc2_debut
	decfsz	cpt_arc15,f	; On passe ici toutes les 30 msec.
	goto	arc2_debut
				; On passe ici apres une courte pause de 1,5  8 secondes.
	movf	varale1s,w
	andlw	0x1F		; 0-31
	addlw	10		; 10-41
	movwf	cpt_arc14	; 10  41 squences lumineuses de suite.
	movlw	9
	movwf	cpt_arc16	; 9 pour 1 millisecondes par boucle.	
	movlw	50
	movwf	cpt_arc17	; cpt_arc17 = 10  101 ms. 1er flash fixe de 50 msec.
	bsf	PORTB,4		; On allume le flash.
	bsf	cpt_arc18,1	; Bit(1)=1 =>  Flash en marche courte dure.
	bsf	cpt_arc18,2	; Bit(2)=1 =>  Flash allum.
	goto	arc2_debut	; On change de mode => Marche + Flash allum (cpt_arc_14, 16, 17).
	

arc2_debut
; -------------------------------------------------------
; Squence arc lectrique 2.
;
; RB5 - Arc lectrique 2
; -------------------------------------------------------
	btfsc	cpt_arc28,0	; Arc 2 actif/inactif longue dure ?
	goto	a2en_ma

	decfsz	cpt_arc21,f 	; Ici, on est en mode INACTIF TRES LONGUE DUREE.
	goto	arc3_debut
	decfsz	cpt_arc22,f	; On passe ici toutes les 38 ms (256*165 s).
	goto	arc3_debut

	movlw	24
	movwf	cpt_arc22

	decfsz	cpt_arc23,f	; On passe ici toutes les secondes.
	goto	arc3_debut
	movlw	1
	movwf	cpt_arc24
	movwf	cpt_arc25
	movlw	ACT_AR2		; 40 = temps actif => 40 secondes actif.
	movwf	cpt_arc23	; cpt_arcx3 =  Temps du prochain fonctionnement longue dure.
	clrf	cpt_arc28	; Bit(1)=0 =>  Flash en arrt courte dure + Bit(2)=0 => Flash teint.
	bsf	cpt_arc28,0	; Bit(0)=1 =>  Retour  la squence => Arc en fonctionnement longue dure.
	goto	arc3_debut	; On change de mode => Mode actif longue dure (cpt_arc_11, 12, 13, 16, 17).

a2en_ma	decfsz	cpt_arc21,f 	; Ici, on est en mode ACTIF TRES LONGUE DUREE.
	goto	a2en_mb
	decfsz	cpt_arc22,f	; On passe ici toutes les 38 ms (256*165 s).
	goto	a2en_mb

	movlw	24
	movwf	cpt_arc22

	decfsz	cpt_arc23,f	; On passe ici toutes les secondes.
	goto	a2en_mb
	movlw	POS_AR2		; 30 = Temps de pause => 30 secondes inactif.
	movwf	cpt_arc23	; cpt_arcx3 =  Temps de la prochaine pause longue dure.
	bcf	PORTB,5		; On teint le flash.
	bcf	cpt_arc28,0	; Bit(0)=0 =>  Retour  la squence => Arc en pause longue dure.
	goto	arc3_debut	; On change de mode => Mode inactif longue dure (cpt_arc_11, 12, 13).

a2en_mb	btfss	cpt_arc28,1
	goto	a2en_me
	btfsc	cpt_arc28,2
	goto	a2en_md	

	decfsz	cpt_arc26,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ETEINT.
	goto	arc3_debut
	movlw	9
	movwf	cpt_arc26	; 9 pour 1,3 millisecondes par boucle.	
	decfsz	cpt_arc27,f
	goto	arc3_debut
	decfsz	cpt_arc24,f	; Fin de courte squence de flash allum.
	goto	a2en_mc
	movf	varale1s,w	; On  fini les squences de flash.	
	andlw	0x1F
	movwf	cpt_arc25	; 0-31
	movf	varale1s+1,w
	andlw	0x0F		; 0-15
	addlw	8		; 8-23
	addwf	cpt_arc25,f	; 8-54 pour 0,3  1,7 secondes de pause(cpt14+cpt15). (255 = 8 sec).
	bcf	cpt_arc28,1	; Bit(1) = 0. 
	goto	arc3_debut	; On change de mode => Flash en arrt courte dure courte dure (cpt_arc_14, 15).

a2en_mc	movf	varale1s,w
	andlw	0x3F		
	movwf	cpt_arc27	; 0-63
	movf	varale1s+1,w
	andlw	0x1F		; 0-31
	addlw	10		; 10-41
	addwf	cpt_arc27,f	; 10-104  cpt_arc27 = 10  104 ms.
	bsf	PORTB,5		; On allume le flash.
	bsf	cpt_arc28,2	; Bit(2) = 1. Flash allume.
	goto	arc3_debut	; On change de mode => Flash allum (cpt_arc_16 + cpt_arc27).

a2en_md	decfsz	cpt_arc26,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ALLUME.
	goto	arc3_debut	
	movlw	9
	movwf	cpt_arc26	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc27,f
	goto	arc3_debut	
	movwf	cpt_arc27	; cpt_arc27 = 10  101 ms.
	movf	varale1s,w
	andlw	0x0F		
	movwf	cpt_arc27	; 0-15
	movf	varale1s+1,w
	andlw	0X07		; 0-7
	addlw	10		; 10-17
	addwf	cpt_arc27,f	; 10-32  cpt_arc27 = 10  32 ms.
	bcf	PORTB,5		; on teint le flash.
	bcf	cpt_arc28,2	; Bit(2) = 0. Flash teint.
	goto	arc3_debut	; On change de mode => Flash teint (cpt_arc_16,17).
	
a2en_me	decfsz	cpt_arc24,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + ARRET COURTE DUREE.
	goto	arc3_debut
	decfsz	cpt_arc25,f	; On passe ici toutes les 30 msec.
	goto	arc3_debut
				; On passe ici apres une courte pause de 1,5  8 secondes.
	movf	varale1s,w
	andlw	0x0F		; 0-15
	addlw	5		; 5-20
	movwf	cpt_arc24	; 5  20 squences lumineuses de suite.
	movlw	9
	movwf	cpt_arc26	; 9 pour 1 millisecondes par boucle.	
	movlw	50
	movwf	cpt_arc27	; cpt_arc27 = 10  101 ms. 1er flash fixe de 50 msec.
	bsf	PORTB,5		; On allume le flash.
	bsf	cpt_arc28,1	; Bit(1)=1 =>  Flash en marche courte dure.
	bsf	cpt_arc28,2	; Bit(2)=1 =>  Flash allum.
	goto	arc3_debut	; On change de mode => Marche + Flash allum (cpt_arc_14, 16, 17).


arc3_debut
	btfsc	mode,1		; Cavalier[2]=0 ? => Arc 3
	goto	feup1_debut	; Cavalier = 1 !
; -------------------------------------------------------
; Squence arc lectrique 3.
;
; RB6 - Arc lectrique 3
; -------------------------------------------------------
	btfsc	cpt_arc38,0	; Arc 2 actif/inactif longue dure ?
	goto	a3en_ma

	decfsz	cpt_arc31,f 	; Ici, on est en mode INACTIF TRES LONGUE DUREE.
	goto	feup1_debut
	decfsz	cpt_arc32,f	; On passe ici toutes les 38 ms (256*165 s).
	goto	feup1_debut

	movlw	25
	movwf	cpt_arc32

	decfsz	cpt_arc33,f	; On passe ici toutes les secondes.
	goto	feup1_debut
	movlw	1
	movwf	cpt_arc34
	movwf	cpt_arc35
	movlw	ACT_AR3		; 40 = temps actif => 40 secondes actif.
	movwf	cpt_arc33	; cpt_arcx3 =  Temps du prochain fonctionnement longue dure.
	clrf	cpt_arc38	; Bit(1)=0 =>  Flash en arrt courte dure + Bit(2)=0 => Flash teint.
	bsf	cpt_arc38,0	; Bit(0)=1 =>  Retour  la squence => Arc en fonctionnement longue dure.
	goto	feup1_debut	; On change de mode => Mode actif longue dure (cpt_arc_11, 12, 13, 16, 17).

a3en_ma	decfsz	cpt_arc31,f 	; Ici, on est en mode ACTIF TRES LONGUE DUREE.
	goto	a3en_mb
	decfsz	cpt_arc32,f	; On passe ici toutes les 38 ms (256*165 s).
	goto	a3en_mb

	movlw	25
	movwf	cpt_arc32

	decfsz	cpt_arc33,f	; On passe ici toutes les secondes.
	goto	a3en_mb
	movlw	POS_AR3		; 30 = Temps de pause => 30 secondes inactif.
	movwf	cpt_arc33	; cpt_arcx3 =  Temps de la prochaine pause longue dure.
	bcf	PORTB,6		; On teint le flash.
	bcf	cpt_arc38,0	; Bit(0)=0 =>  Retour  la squence => Arc en pause longue dure.
	goto	feup1_debut	; On change de mode => Mode inactif longue dure (cpt_arc_11, 12, 13).

a3en_mb	btfss	cpt_arc38,1
	goto	a3en_me
	btfsc	cpt_arc38,2
	goto	a3en_md	

	decfsz	cpt_arc36,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ETEINT.
	goto	feup1_debut	
	movlw	9
	movwf	cpt_arc36	; 9 pour 1,3 millisecondes par boucle.	
	decfsz	cpt_arc37,f
	goto	feup1_debut
	decfsz	cpt_arc34,f	; Fin de courte squence de flash allum.
	goto	a3en_mc
	movf	varale1s,w	; On  fini les squences de flash.	
	andlw	0x1F		; 0-31
	addlw	4		; 4-35
	movwf	cpt_arc35	; 4-35 pour 0,1  1,1 secondes de pause de courte dure (cpt14+cpt15). (255 = 8 sec).
	bcf	cpt_arc38,1	; Bit(1) = 0. 
	goto	feup1_debut	; On change de mode => Flash en arrt courte dure (cpt_arc_14, 15).

a3en_mc	movf	varale1s,w
	andlw	0x3F		
	movwf	cpt_arc37	; 0-63
	movf	varale1s+1,w
	andlw	0x1F		; 0-31
	addlw	10		; 10-41
	addwf	cpt_arc37,f	; 10-104  cpt_arc37 = 10  104 ms.
	bsf	PORTB,6		; On allume le flash.
	bsf	cpt_arc38,2	; Bit(2) = 1. Flash allume.
	goto	feup1_debut	; On change de mode => Flash allum (cpt_arc_16 + cpt_arc37).

a3en_md	decfsz	cpt_arc36,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + MARCHE COURTE DUREE + FLASH ALLUME.
	goto	feup1_debut	
	movlw	9
	movwf	cpt_arc36	; 9 pour 1 millisecondes par boucle.	
	decfsz	cpt_arc37,f
	goto	feup1_debut	
	movwf	cpt_arc37	; cpt_arc37 = 10  101 ms.
	movf	varale1s,w
	andlw	0x0F		
	movwf	cpt_arc37	; 0-15
	movf	varale1s+1,w
	andlw	0X07		; 0-7
	addlw	10		; 10-17
	addwf	cpt_arc37,f	; 10-32  cpt_arc37 = 10  32 ms.
	bcf	PORTB,6		; on teint le flash.
	bcf	cpt_arc38,2	; Bit(2) = 0. Flash teint.
	goto	feup1_debut	; On change de mode => Flash teint (cpt_arc_16,17).
	
a3en_me	decfsz	cpt_arc34,f	; Ici, on est en mode ACTIF TRES LONGUE DUREE + ARRET COURTE DUREE.
	goto	feup1_debut	
	decfsz	cpt_arc35,f	; On passe ici toutes les 30 msec.
	goto	feup1_debut	
				; On passe ici apres une courte pause de 1,5  8 secondes.
	movf	varale1s,w
	andlw	0x07		; 0-15
	addlw	3		; 3-18
	movwf	cpt_arc34	; 3  18 squences lumineuses de suite.
	movlw	9
	movwf	cpt_arc36	; 9 pour 1 millisecondes par boucle.	
	movlw	50
	movwf	cpt_arc37	; cpt_arc37 = 10  101 ms. 1er flash fixe de 50 msec.
	bsf	PORTB,6		; On allume le flash.
	bsf	cpt_arc38,1	; Bit(1)=1 =>  Flash en marche courte dure.
	bsf	cpt_arc38,2	; Bit(2)=1 =>  Flash allum.
				; On change de mode => Marche + Flash allum (cpt_arc_14, 16, 17).



feup1_debut
	btfss	mode,2		; Cavalier[2]=1 ? => Feu de pntration n1.
	goto	feup23_debut	; Cavalier = 0 !
; -------------------------------------------------------------------------------
; Feux de pntration.
;
;   RC5 - Feu de de pntration n1 indpendant 7 secondes de flash + 20 secondes de pause.
;   RC2 - Feu de de pntration n2 altern
;   RB6 - Feu de de pntration n3 altern
;
;   cpt_feup11	; Compteur de boucle principale.
;   cpt_feup12	; Compteur de boucle secondaire.
;   cpt_feup13	; Compteur ralentisseur de boucle.
;   cpt_feup14	; =0 => 1er flash, =1 => 2me flash, =2 => 3me flash, 4=> Pause longue dure
;   cpt_feup15	; Compteur de boucle en pause.
;   cpt_feup16  ; 2, 4, 6, 8, 10 => Squence 1 / 2 / 3 / 4 / 5 flashs de suite.
;
; Squence 2 :  -        -        -        -        -        -        -        -
;          4 :  - -              - -              - -              - -
;          6 :  - - -               - - -               - - -               - - -
; -------------------------------------------------------------------------------
;
	decfsz	cpt_feup13	
	goto	feup23_debut
	movlw	8
	movwf	cpt_feup13	; Pour ralentir cette boucle

	decfsz	cpt_feup11	
	goto	feup23_debut

	movf	cpt_feup14,w	
	sublw	255
	btfss	STATUS,Z	; Pause longue dure ?
	goto	feup1a
	decfsz	cpt_feup12	; Pause longue. On passe ici toutes les 256 * 165 S = 42 msec.
	goto	feup23_debut

	movlw	3
	movwf	cpt_feup12
	decfsz	cpt_feup15	; Pause longue. On passe ici toutes les secondes.
	goto	feup23_debut	; 3 * 9 * 256 * 165 S = 1 sec.
	clrf	cpt_feup14	; Fin de pause, on repasse en squence 0 : 2 => cpt_feup16.
	movlw	NB_PE1		; Squence lumineuse de 90 flashs = 30 sec.
	movwf	cpt_feup15
	movlw	2
	addwf	cpt_feup16,f	; 2, 4, 6, 8, 10 => Squence 2 / 3 / 4 / 5 flashs de suite.
	movlw	12
	subwf	cpt_feup16,w	; 12 retour  la squence 2 flashs de suite.
	btfss	STATUS,Z
	goto	feup23_debut
	movlw	2
	movwf	cpt_feup16
	goto	feup23_debut
feup1a	btfsc	cpt_feup14,0	
	goto	feup1b	
	bsf	PORTC,5		; Allume le feu, cpt_feup14 = 0,2,4,6,8,10...
	movlw	2		; pendant 2 * 9 * 165 S = 3 msec.
	movwf	cpt_feup11	
	incf	cpt_feup14,f
	goto	feup23_debut
feup1b	bcf	PORTC,5		; Eteint le feu, cpt_feup14 = 1,3,5,7,11...
	movlw	70		; pendant 70 * 9 * 165 S =  100 msec.
	movwf	cpt_feup11	
	incf	cpt_feup14,f
	movf	cpt_feup14,w
	subwf	cpt_feup16,w	; 2, 4, 6, 8, 10 => Squence 2 / 3 / 4 / 5 flashs de suite.
	btfss	STATUS,Z	; Sequence n 8 ?
	goto	feup23_debut

	movlw	253		; Pause courte dure de 253 * 9 * 165 S = 400 msec.
	movwf	cpt_feup11	
	clrf	cpt_feup14	; Raz compteur de squence de flash.
	decfsz	cpt_feup15
	goto	feup23_debut

	decf	cpt_feup14,f	; Si l'on a fait 256 flashs = 2 mn, cpt_feup14 = 255 => Pause longue dure.
	movlw	POS_PE1		; 15 secondes de pause longue.
	movwf	cpt_feup15
	movlw	2
	movwf	cpt_feup12	; Pour la premire boucle de 1 sec = 2 * 9 * 42 ms.

feup23_debut
	btfss	mode,1		; Cavalier[1]=1 ? => Feux de pntration n2 et n3.
	goto	clrap1_debut	; Cavalier = 0 !

	decfsz	cpt_feup23	
	goto	clrap1_debut
	movlw	8
	movwf	cpt_feup23	; Pour ralentir cette boucle

	decfsz	cpt_feup21	
	goto	clrap1_debut

	movf	cpt_feup24,w	
	sublw	255
	btfss	STATUS,Z	; Pause longue dure ?
	goto	feup23a
	decfsz	cpt_feup22	; Pause longue. On passe ici toutes les 256 * 165 S = 42 msec.
	goto	clrap1_debut
	movlw	3
	movwf	cpt_feup22
	decfsz	cpt_feup25	; Pause longue. On passe ici toutes les secondes.
	goto	clrap1_debut	; 3 * 9 * 256 * 165 S = 1 sec.
	clrf	cpt_feup24	; Fin de pause, on repasse en squence 0 : 2 => cpt_feup26.
	movlw	NB_PE23		; Squence lumineuse de 120 flashs = 40 sec.
	movwf	cpt_feup25
	movlw	2
	addwf	cpt_feup26,f	; 2, 4, 6, 8, 10 => Squence 2 / 3 / 4 / 5 flashs de suite.
	movlw	12
	subwf	cpt_feup26,w	; 12 retour  la squence 2 flashs de suite.
	btfss	STATUS,Z
	goto	clrap1_debut
	movlw	2
	movwf	cpt_feup26
	goto	clrap1_debut

feup23a	btfsc	cpt_feup24,0	
	goto	feup23b	
	btfss	cpt_feup25,0	
	bsf	PORTC,2		; Allume le feu, cpt_feup24 = 0,2,4,6,8,10...
	btfsc	cpt_feup25,0	
	bsf	PORTB,6		; Allume le feu, cpt_feup24 = 0,2,4,6,8,10...

	movlw	2		; pendant 2 * 9 * 165 S = 3 msec.
	movwf	cpt_feup21	
	incf	cpt_feup24,f
	goto	clrap1_debut

feup23b	btfss	cpt_feup25,0	
	bcf	PORTC,2		; Eteint le feu, cpt_feup24 = 1,3,5,7,11...
	btfsc	cpt_feup25,0	
	bcf	PORTB,6		; Eteint le feu, cpt_feup24 = 1,3,5,7,11...
	movlw	70		; pendant 70 * 9 * 165 S =  100 msec.
	movwf	cpt_feup21	
	incf	cpt_feup24,f
	movf	cpt_feup24,w
	subwf	cpt_feup26,w	; 2, 4, 6, 8, 10 => Squence 2 / 3 / 4 / 5 flashs de suite.
	btfss	STATUS,Z	; Sequence n 8 ?
	goto	clrap1_debut

	movlw	253		; Pause courte dure de 253 * 9 * 165 S = 400 msec.
	movwf	cpt_feup21	
	clrf	cpt_feup24	; Raz compteur de squence de flash.
	decfsz	cpt_feup25
	goto	clrap1_debut

	decf	cpt_feup24,f	; Si l'on a fait 256 flashs = 2 mn, cpt_feup24 = 255 => Pause longue dure.
	movlw	POS_PE23	; 20 secondes de pause longue.
	movwf	cpt_feup25
	movlw	2
	movwf	cpt_feup22	; Pour la premire boucle de 1 sec = 2 * 9 * 42 ms.


clrap1_debut
; ---------------------------------------------------------------------------------------------------
; Clignotants rapides
;
;   RC4 - Clignotant rapide 1
;   RC3 - Clignotant rapide 2, 10 % plus rapide que le n1.
;   RB7 - Clignotant rapide 3, 20 % plus rapide que le n1. Si Cavalier[3]=0.
;
;   cpt_clrap11	; Compteur de boucle principale.
;   cpt_clrap12	; Compteur de boucle principale. Fonctionne 35 secondes, puis pause de 35 secondes.
;   cpt_clrap21	; Compteur de boucle principale.
;   cpt_clrap22	; Compteur de boucle principale. Fonctionne 35 secondes, puis pause de 35 secondes.
;   cpt_clrap31	; Compteur de boucle principale.
;   cpt_clrap32	; Compteur de boucle principale. Fonctionne 35 secondes, puis pause de 35 secondes.
; ----------------------------------------------------------------------------------------------------
;
	movf	cpt_flam02,w	; cpt_flam02 = de 0  17. Utilise ici le compteur de flamme 1 toujours actif pour conomiser un compteur.
	sublw	5		; Pour passer dans la squence quand cpt_flam02 = 5 pour ne pas passer ici en mme temps que dans d'autres squences cycliques.
	btfss	STATUS,Z
	goto	anim_fin	; On ne passe dans aucune squences de clignotants.
	decfsz	cpt_clrap11
	goto	clrap2a	
	movlw	VIT_CLR1	; 2 * X * 17 * 165 s. Priode de clignotement = 280 msec.
	addwf	can_lue2,w
	movwf	cpt_clrap11
	movlw	b'00010000'
	btfss	cpt_gene00,5	; Bit 5 du mot "cpt_gene00" pour clignotant actif ou teint.
	xorwf	PORTC,f		; Clignotant rapide 1.
	decfsz	cpt_clrap12
	goto	clrap2a	
	btfsc	cpt_gene00,5	; Si cpt_clrap12 = 0 => on bascule de mode actif ou teint.
	goto	clrap1b
	bsf	cpt_gene00,5
	bcf	PORTC,4		; Si on passse en mode teint, on teint la led.
	goto	clrap2a	
clrap1b	bcf	cpt_gene00,5

clrap2a	decfsz	cpt_clrap21
	goto	clrap3a	
	movlw	VIT_CLR2	; 2 * X 17 * 165 s. Priode de clignotement = 258 msec.
	addwf	can_lue2,w
	movwf	cpt_clrap21
	movlw	b'00001000'
	btfss	cpt_gene00,6	; Bit 6 du mot "cpt_gene00" pour clignotant actif ou teint.
	xorwf	PORTC,f		; Clignotant rapide 2.
	decfsz	cpt_clrap22
	goto	clrap3a	
	btfsc	cpt_gene00,6	; Si cpt_clrap22 = 0 => on bascule de mode actif ou teint.
	goto	clrap2b 
	bsf	cpt_gene00,6
	bcf	PORTC,3		; Si on passse en mode teint, on teint la led.
	goto	clrap3a	
clrap2b	bcf	cpt_gene00,6

clrap3a	btfss	mode,3		; Cavalier[3]=1 ? => Clignotant n 3.
	goto	anim_fin	; Cavalier = 0 !
	decfsz	cpt_clrap31
	goto	anim_fin
	movlw	VIT_CLR3	; 2 * X * 17 * 165 s. Priode de clignotement = 235 msec.
	addwf	can_lue2,w
	movwf	cpt_clrap31
	movlw	b'10000000'
	btfss	cpt_gene00,7	; Bit 7 du mot "cpt_gene00" pour clignotant actif ou teint.
	xorwf	PORTB,f		; Clignotant rapide 3.
	decfsz	cpt_clrap32
	goto	anim_fin
	btfsc	cpt_gene00,7	; Si cpt_clrap32 = 0 => on bascule de mode actif ou teint.
	goto	clrap3b
	bsf	cpt_gene00,7
	bcf	PORTB,7		; Si on passse en mode teint, on teint la led.
	goto	anim_fin
clrap3b	bcf	cpt_gene00,7


anim_fin
	goto	boucle
;
;*******************************************************************
;*******************************************************************
;*******************************************************************
; Entre w = Numro d'effet pour les flammes (0  13).
; Retour w = Nombre de boucles pour les flammes.
;
; Utilisation de la table :
; 	movlw	0x05
;	movwf	PCLATH
;	movf	cpt_flam01,w
;	call	table_flam
;
	ORG 0x600		; Adresse des tableaux en fin de mmoire (Moins de 256 octets par table).
table_flam:
	addwf	PCL,f		; Ajouter w  PCL
	retlw	120		;  0 Normal
	retlw	48		;  1 Rotation
	retlw	120		;  2 Normal
	retlw	60		;  3 Rapide
	retlw	120		;  4 Normal
	retlw	41		;  5 Puissant
	retlw	120		;  6 Normal
	retlw	225		;  7 Disparait + Rapparait lentement (en n 11). 225 obligatoire.
	retlw	80		;  8 Normal
	retlw	129		;  9 Disparait + Rapparait rapidement (en n 13). 129 obligatoire.
	retlw	80		; 10 Normal
	retlw	225		; 11 Rapparait lentement (suite du n 7) 225 obligatoire.
	retlw	80		; 12 Normal
	retlw	129		; 13 Rapparait rapidement en (suite du n 9). 129 obligatoire.
	retlw	0		; 14 Fin de table
	retlw	0		; 15 Fin de table



;*******************************************************************
; Entre w = Compteur de squence pour 3 tubes fluorescents
; Retour w = Valeurs des 3 lampes = XXXXX210 (Bit = 1 = lampe allume)
;          = Bit(7)= 1 = fin de table => Faire raz compteur.
;
; Tube Fluo 1 = Nombreuses tentatives d'allumages puis reste allum un certain temps.
; Tube Fluo 2 = Nombreuses tentatives d'allumages mais reste allum qu'un petit moment.
; Tube Fluo 3 = Tentatives d'allumages en continu, ne parvient pas  s'allumer.
;
; Utilisation de la table :
; 	movlw	0x07
;	movwf	PCLATH
;	movf	cpt_fluo,w	
;	call	table_fluo
;
	ORG 0x700		; Adresse des tableaux en fin de mmoire (moins de 256 octets par table)
table_fluo:
	addwf	PCL,f		; Ajouter w  PCL
	retlw	b'00000111'	; Retour : Lampes: X,X,X,X, lampe n 3, lampe n 2, lampe n  1.
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000110'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000110'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000100'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000101'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000110'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000100'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000111'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000110'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000010'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000100'
	retlw	b'00000010'
	retlw	b'00000000'
	retlw	b'00000101'
	retlw	b'00000110'
	retlw	b'00000110'
	retlw	b'00000100'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000000'
	retlw	b'00000000'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000101'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000111'
	retlw	b'00000101'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000111'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000101'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000001'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000011'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'
	retlw	b'00000111'	; Il faut que les dernires valeurs = '0' pour que les tubes soient teints pendant la pause longue.
	retlw	b'10000000'	; Dernire valeur valide. Fin de table => Bit(7) = 1.



;***************************************************************************************************************
; Dbut du code d'initialisation en page n1, 0x900 pour laisser de la place au programme
; principal en page n0 0x000-0x7FF.
; Attention, ici les 'call' ne doivent faire appel qu' des sous-programmes en page n1.
;
	ORG     0x900		; Le code dbute  l'origine de la page n1
init_suite
	BANK0
	movlw   b'00100000'     ; Interruptions = GIE=0 pour le moment, T0IE=1 (Timer overflow interrupt enable bit).
	movwf   INTCON
	movlw   b'00000001'     ; Format de conversion analogique 8 bits les plus forts sur un octet + Vref=+5V + Convertisseur=actif.
	movwf   ADCON0
;
	BANK1
	movlw   b'01110000'     ; 'x111'=Vitesse 8 Mhz + 'x'=OSTS interne + 'x'=HFINTOSC is stable + 'x'=LFINTOSC is stable + '0'=SCS interne.
	movwf	OSCCON		; 'x' = Bit en lecture seulement.
	movlw   b'00100000'     ; Dure de conversion analogique = 2 s = F/16 pour 8 Mhz.
	movwf   ADCON1
	BANK0
;
	clrf	PORTA           ; Au dpart, teint toutes les sorties.
	clrf	PORTB
	clrf	PORTC
;
;   T0CS   =   0 = Source interne pour TMR0
;   PSA    =   0 = Prdiviseur pour le Timer0
;   PS     = 001 = Prdiviseur pour Timer0 = 1/4 => (4*4)/8 Mhz = 2 sec
;
	BANK1
	movlw   b'11010001'     ; PortB, pas de rsistances de rappel, Timer0 Clock Source Select bit = Focs/4,
	movwf	OPTION_REG	; Prescaler is assigned to the Timer0 module.

	movlw   b'00110111'     ; '1'=Weak pull-up actif.
	movwf	WPUA

	clrf	PIE1	 	; '0'=Pas d'interruption.
	clrf	PIE2
	clrf	IOCA

	movlw   b'00001001'     ; PortA en sortie (4 bits) sauf RA0=Entre CAN0 et RA3=Entre numrique.
	movwf   TRISA

	clrf	TRISB		; PortB en sortie (4 bits).
	clrf	TRISC		; PortC en sortie (8 bits).
;
	BANK2
	movlw   b'00000001'     ; Utilisation de RA0 en entre analogique CAN0.
	movwf	ANSEL

	clrf	ANSELH		; Utilisation de RA0 en entre analogique CAN0.
	clrf	CM1CON0		; Les comparateurs ne sont pas utiliss.
	clrf	CM2CON0	
        BANK0


;*******************************************************************
; Initialisation des variables du programme.
;	
	movlw 0x20 		; Raz de toutes les variables.
	movwf FSR		; On charge l'adresse de dbut (0x20) de la zone des variables.
clr_var	clrf INDF		; Effacement du contenu du registre point par FSR
	incf FSR,f		; On incrmente l'adresse.
	movlw 0x80		; Adresse de fin + 1 (0x80)
	subwf FSR,w
	btfss STATUS,Z		; On teste si (FSR) = 0x80.
	goto clr_var		; Non : on recommence.

	movlw	1		; 1 => Pour passer tout de suite dans la boucle.
	movwf	cpt_fluo11
	movwf	cpt_fluo12
	movwf	cpt_fluo13

	movwf	cpt_fluo21
	movwf	cpt_fluo22
	movwf	cpt_fluo23

	movwf	cpt_fluo31
	movwf	cpt_fluo32
	movwf	cpt_fluo33

	movwf	cpt_flam01
	movwf	cpt_flam02
	movwf	cpt_flam03

	movwf	cpt_arc11
	movwf	cpt_arc12
	movwf	cpt_arc13	; Fin des boucles de pause longue dure.
	movwf	cpt_arc21
	movwf	cpt_arc22
	movwf	cpt_arc23	; Fin des boucles de pause longue dure.
	movwf	cpt_arc31
	movwf	cpt_arc32
	movwf	cpt_arc33	; Fin des boucles de pause longue dure.

	movwf	cpt_flash13
	movwf	cpt_flash23
	movwf	cpt_flash33

	movwf	cpt_feup11
	movwf	cpt_feup13
	movwf	cpt_feup22
	
	movwf	cpt_lent03		

	movwf	cpt_clrap11
	movwf	cpt_clrap21		
	movwf	cpt_clrap31

	movlw	4		; Pour dcaler la squence pene1 et pene23.
	movwf	cpt_feup23

	movlw	85		; Pour dcaler la squence marche/arrt du clignotant n 2.
	movwf	cpt_clrap22
	movlw	170		; Pour dcaler la squence marche/arrt du clignotant n 3.
	movwf	cpt_clrap32

	movlw	90		; Squence lumineuse de 90 flashs = 30sec.
	movwf	cpt_feup15
	movlw	2
	movwf	cpt_feup16	; 2 => Squence 1 flash au dpart.

	movlw	120		; Squence lumineuse de 120 flashs = 40 sec.
	movwf	cpt_feup25
	movlw	2
	movwf	cpt_feup26	; 2 => Squence 1 flash au dpart.

	movlw	b'00001110'
	movwf	cpt_fluo33	; Tubes fluo en marche.

	movlw	0x30		; Pour initialiser les gnrateurs de nombre alatoire.
	movwf	varale1s
	movwf	varale2s+1
	movlw	0x45
	movwf	varale1s+1
	movwf	varale2s

;
;***********************************************************************************************
; Lecture des cavaliers en entre de R3 et sorties RA5, RA4, RC5 et RC4.
;
	clrf	mode
	bsf	PORTC,4		; Lecture des cavaliers pour choisir le mode de fonctionnement.
	clrf	temp1		; Les autres sorties sont dj  0.
	decfsz	temp1,f
	goto	$-1             ; Petite tempo pour que l'entre multiplexe se stabilise.
	btfsc	PORTA,3
	bsf	mode,0          ; 0-0-0-1.
;
	bcf	PORTC,4
	bsf	PORTC,5
	decfsz	temp1,f
	goto	$-1             ; Petite tempo pour que l'entre multiplexe se stabilise.
	btfsc	PORTA,3
	bsf	mode,1 		; 0-0-1-0.
;	
	bcf	PORTC,5
	bsf	PORTA,4
	decfsz	temp1,f
	goto	$-1             ; Petite tempo pour que l'entre multiplexe se stabilise.
	btfsc	PORTA,3
	bsf	mode,2 		; 0-1-0-0.
;
	bcf	PORTA,4
	bsf	PORTA,5
	decfsz	temp1,f
	goto	$-1             ; Petite tempo pour que l'entre multiplexe se stabilise.
	btfsc	PORTA,3
	bsf	mode,3		; 1-0-0-0.
;
	bcf	PORTA,5		; Au dpart, teint toutes les sorties.
;
	btfss	mode,0		; Cavalier[0]=1 ? => Clignotants rapides.
	goto	nocli45		; Cavalier = 0 !
	movlw	b'00000010'
	xorwf	PORTA,f		; Allume une des deux lampes de clignotant rapide et altrn.

nocli45	bsf	ADCON0,GO	; Lance la conversion can pour la prochaine lecture.
	return
;
	end